import io
import tarfile

from acquire.crypt import EncryptedStream
from acquire.outputs.base import Output


class TarOutput(Output):
    def __init__(self, path, compress=False, encrypt=False, public_key=None, **kwargs):
        ext = ".tar" if ".tar" not in path.suffixes else ""
        mode = "w|" if encrypt else "w:"

        if compress:
            ext += ".gz" if ".gz" not in path.suffixes else ""
            mode += "gz"

        if encrypt:
            ext += ".enc"

        self._fh = None
        self.path = path.with_suffix(path.suffix + ext)

        if encrypt:
            self._fh = EncryptedStream(self.path.open("wb"), public_key)
            self.tar = tarfile.open(fileobj=self._fh, mode=mode)
        else:
            self.tar = tarfile.open(name=self.path, mode=mode)

    def write(self, path, fh, size=None, entry=None):
        size = size or getattr(fh, "size", None)
        if size is None:
            offset = fh.tell()
            fh.seek(0, io.SEEK_END)
            size = fh.tell()
            fh.seek(offset)

        info = self.tar.tarinfo()
        info.name = path
        info.uname = "root"
        info.gname = "root"
        info.size = size

        self.tar.addfile(info, fh)

    def close(self):
        self.tar.close()
        if self._fh:
            self._fh.close()



from pathlib import Path
from typing import List, Optional, Protocol, runtime_checkable


@runtime_checkable
class UploaderPlugin(Protocol):
    """Creates a typing definition to which an UploaderPlugin should adhere."""

    def upload_files(self, paths: List[Path], proxies: Optional[dict[str, str]] = None) -> None:
        """Uploads the files in ``paths`` to a destination.

        Args:
            paths: A list of files to upload.
            proxies: Proxies used as an intermediate during an upload.
        """
        ...



from __future__ import annotations

import ctypes
from ctypes.wintypes import BOOL, DWORD, HANDLE, LPWSTR, PHANDLE, PULONG, ULONG, USHORT

PVOID = ctypes.c_void_p
NTSTATUS = ULONG
NULL = None


class UNICODE_STRING(ctypes.Structure):
    _fields_ = [
        ("Length", USHORT),
        ("MaximumLength", USHORT),
        ("Buffer", LPWSTR),
    ]

    def __str__(self) -> str:
        return self.Buffer

    @classmethod
    def from_str(cls, value: str) -> None:
        """Initializes a UNICODE_STRING structure."""
        destination = cls()
        value_buffer = ctypes.create_unicode_buffer(value)

        ctypes.memset(ctypes.addressof(destination), 0, ctypes.sizeof(destination))
        destination.Buffer = ctypes.cast(value_buffer, LPWSTR)
        destination.Length = ctypes.sizeof(value_buffer) - 2  # Excluding terminating NULL character
        destination.MaximumLength = destination.Length

        return destination


PUNICODE_STRING = ctypes.POINTER(UNICODE_STRING)


class OBJECT_DIRECTORY_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("Name", UNICODE_STRING),
        ("TypeName", UNICODE_STRING),
    ]

    @property
    def name(self) -> str:
        return str(self.Name)

    @property
    def type_name(self) -> str:
        return str(self.TypeName)


__all__ = [
    "BOOL",
    "DWORD",
    "HANDLE",
    "LPWSTR",
    "PHANDLE",
    "PULONG",
    "ULONG",
    "USHORT",
    "PVOID",
    "NTSTATUS",
    "NULL",
    "UNICODE_STRING",
    "PUNICODE_STRING",
    "OBJECT_DIRECTORY_INFORMATION",
]
